-- LSP Config

local lsp = {}

local lsp_attach = function(client, buf)
	vim.api.nvim_buf_set_option(buf, "formatexpr", "v:lua.vim.lsp.formatexpr()")
	vim.api.nvim_buf_set_option(buf, "omnifunc", "v:lua.vim.lsp.omnifunc")
	vim.api.nvim_buf_set_option(buf, "tagfunc", "v:lua.vim.lsp.tagfunc")

	local bufopts = { noremap=true, silent=true, buffer=buf }
	vim.keymap.set('n', 'gD', vim.lsp.buf.declaration, bufopts)
	vim.keymap.set('n', 'gd', vim.lsp.buf.definition, bufopts)
	vim.keymap.set('n', 'K', vim.lsp.buf.hover, bufopts)
	vim.keymap.set('n', 'gi', vim.lsp.buf.implementation, bufopts)
	vim.keymap.set('n', '<C-k>', vim.lsp.buf.signature_help, bufopts)
	vim.keymap.set('n', '<space>D', vim.lsp.buf.type_definition, bufopts)
	vim.keymap.set('n', '<space>rn', vim.lsp.buf.rename, bufopts)
	vim.keymap.set('n', '<space>ca', vim.lsp.buf.code_action, bufopts)
	vim.keymap.set('n', 'gr', vim.lsp.buf.references, bufopts)
end

function lsp.init()
	require'mason'.setup()
	require'mason-lspconfig'.setup()

	local capabilities = require('cmp_nvim_lsp').update_capabilities(vim.lsp.protocol.make_client_capabilities())
	local lspconfig = require'lspconfig'

	lspconfig.sumneko_lua.setup {
		capabilities = capabilities,
		settings = {
			Lua = {
				diagnostics = {
					globals = { 'vim' }
				}
			}
		},
		on_attach = lsp_attach
	}
	lspconfig.tsserver.setup {
		capabilities = capabilities,
		on_attach = lsp_attach
	}
	lspconfig.gopls.setup {
		capabilities = capabilities,
		on_attach = lsp_attach
	}
	--lspconfig.rust_analyzer.setup {
	--	capabilities = capabilities,
	--	on_attach = lsp_attach
	--}
	require("rust-tools").setup({
		server = {
			capabilities = capabilities,
			on_attach = lsp_attach,
			standalone = false
		}
	})
end

return lsp
